# simple illustration client/server pair; client program sends a string
# to server, which echoes it back to the client (in multiple copies),
# and the latter prints to the screen
# this is the server

import socket
import sys
#import objc
import time
import threading
#from AppKit import NSScreen

if sys.platform == 'darwin':
	
	from Quartz import CGPostMouseEvent, CGWarpMouseCursorPosition, CGDisplayPixelsHigh, CGDisplayPixelsWide
	
	from AppKit import NSEvent
	
	class PyMouse():

		def press(self, x, y, button = 1):
			button_list = [0, 0, 0]
	
			button_list[button - 1] = 1
			
			CGPostMouseEvent((x, y), 1, 3, *button_list)

		def release(self, x, y, button = 1):
	
			CGPostMouseEvent((x, y), 1, 3, 0, 0, 0)

		def move(self, x, y):
			
			CGWarpMouseCursorPosition((float(x), float(y)))
		
		def position(self):

			loc = NSEvent.mouseLocation()
		
			return loc.x, CGDisplayPixelsHigh(0) - loc.y

		def screen_size(self):
			
			return CGDisplayPixelsWide(0), CGDisplayPixelsHigh(0)


elif sys.platform == 'win32':
	
	clicks = [None, 2, 8, 32]

	releases = [None, 4, 16, 64]
	
	from ctypes import *

	from win32api import GetSystemMetrics
	
	PUL = POINTER(c_ulong)
	
	class MouseInput(Structure):

		_fields_ = [("dx", c_long), ("dy", c_long), ("mouseData", c_ulong), ("dwFlags", c_ulong), ("time",c_ulong), ("dwExtraInfo", PUL)]
	
	class Input_I(Union):

		_fields_ = [("mi", MouseInput)]
	
	class Input(Structure):
		
		_fields_ = [("type", c_ulong), ("ii", Input_I)]	
	FInputs = Input * 2

	extra = c_ulong(0)
	
	click = Input_I()
	
	click.mi = MouseInput(0, 0, 0, 2, 0, pointer(extra))
	
	release = Input_I()

	release.mi = MouseInput(0, 0, 0, 4, 0, pointer(extra))

	blob = FInputs( (0, click), (0, release) )

	class POINT(Structure):
	
		_fields_ = [("x", c_ulong), ("y", c_ulong)]

	class PyMouse():

		def press(self, x, y, button = 1):
		
			self.move(x, y)
	
			blob[0].ii.mi.dwFlags = clicks[button]
	
			windll.user32.SendInput(2,pointer(blob),sizeof(blob[0]))

		def release(self, x, y, button = 1):
	
			self.move(x, y)
		
			blob[1].ii.mi.dwFlags = releases[button]
	
			windll.user32.SendInput(2,pointer(blob),sizeof(blob[0]))
	
		def move(self, x, y):

			windll.user32.SetCursorPos(x, y)
	
		def position(self):
			pt = POINT()
			windll.user32.GetCursorPos(byref(pt))

			return pt.x, pt.y
	
		def screen_size(self):

			width = GetSystemMetrics(0)

			height = GetSystemMetrics(1)

			return width, height

else:
	
	from Xlib.display import Display
	
	from Xlib import X
	
	from Xlib.protocol import event
	
	import Xlib.ext.xtest

	display = Display(":0")
	
	root = display.screen().root

	class PyMouse():
		
		def press(self, x, y, button = 1):
			
			focus = display.get_input_focus().focus
			
			rel = focus.translate_coords(root, x, y)
			
			button_list = [None, X.Button1, X.Button3, X.Button2]
			
			try:
				
				mousePress = event.ButtonPress(time=X.CurrentTime, root=root, window=focus, same_screen=1, child=X.NONE, root_x=x, root_y=y, event_x=rel.x, event_y=rel.y, state=0, detail=button_list[button])
			
				focus.send_event(mousePress)
			
			except:
			
				pass
			display.sync()
		
		def release(self, x, y, button = 1):
			
			focus = display.get_input_focus().focus
			
			rel = focus.translate_coords(root, x, y)
			
			button_list = [None, X.Button1, X.Button3, X.Button2]
			
			try:
				
				mouseRealease = event.ButtonRelease(time=X.CurrentTime, root=root, window=focus, same_screen=1, child=X.NONE, root_x=x, root_y=y, event_x=rel.x, event_y=rel.y, state=1, detail=button_list[button])
	
				focus.send_event(mouseRealease)
			
			except:
				
				pass
		
			display.sync()
	
		def click(self, x, y, button = 1):
			try:
				self.press(x, y, button)
				self.release(x, y, button)
			except:
				# Using xlib-xtest fake input
				self.move(x, y)
				Xlib.ext.xtest.fake_input (display, X.ButtonPress, button+1)
			display.sync()

		def move(self, x, y):
			root.warp_pointer(x, y)
			display.sync()
		def position(self):
			coord = display.screen().root.query_pointer()._data
			return coord["root_x"], coord["root_y"]
		def screen_size(self):
			width = display.screen().width_in_pixels
			height = display.screen().height_in_pixels
			return width, height

# create a socket
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
# associate the socket with a port
host = '' # can leave this blank on the server side
port = int(sys.argv[1])
s.bind((host, port))
print "Listening for connection... ",

# accept "call" from client
s.listen(1)
global conn
conn, addr = s.accept()
print 'client is at', addr

# read string from client (assumed here to be so short that one call to
# recv() is enough), and make multiple copies (to show the need for the
# "while" loop on the client side)
global pm
global l3x
global l3y
global lx
global ly

pm = PyMouse()
width, height = pm.screen_size()
l3x = [width/2, width/2, width/2]
l3y = [height/2, height/2, height/2]
lx=l3x[0]
ly=l3y[0]

def average(a):
	b = len(a)
	c = 0
	for i in a:
		c = c+i
	return c/b


class Serverservice (threading.Thread):
	def stop(self):
		self.ended = True
	def run (self):
		self.ended=False
		global l3x
		global l3y
		global conn
		while not self.ended:
			data = conn.recv(1024)
			[x, y, z] = eval(data.split("\n")[-2])
			l3x = l3x[1:]
			l3y = l3y[1:]
			l3x.append(int(x))
			l3y.append(int(y))

class Mousecontrol (threading.Thread):
	def stop(self):
		self.ended = True
	def run (self):
		self.ended = False
		global l3x
		global l3y
		global lx
		global ly
		global pm
		while not self.ended:
			#smooth inputs by averaging last 3 readings
			x = (lx*8+(average(l3x)/800.0+0.5)*height)/9
			y = (ly*8+(average (l3y)/1400.0+0.5)*width)/9
			pm.move(y,x)
			lx = x
			ly = y
			time.sleep(0.05)


ss = Serverservice()
ss.__init__()
ss.start()
mc = Mousecontrol()
mc.__init__()
mc.start()
#time.sleep(20)
try:
	q = raw_input()
except:
	ss.stop()
	mc.stop()